@@ -16,7 +16,7 @@
 # You should have received a copy of the GNU General Public License along
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-"""Magicicada."""
+"""Magicicada GTK UI."""
 
 import logging
 import os
@@ -31,14 +31,15 @@ import gtk
 # this shouldn't crash if not found as it is simply used for bug reporting
 try:
     import LaunchpadIntegration
-    launchpad_available = True
+    LAUNCHPAD_AVAILABLE = True
 except ImportError:
-    launchpad_available = False
+    LAUNCHPAD_AVAILABLE = False
 
 from twisted.internet import gtk2reactor  # for gtk-2.0
 gtk2reactor.install()
 
-from magicicada import syncdaemon, dbusiface, logger as logger_helper
+from magicicada import syncdaemon, logger as logger_helper
+from magicicada.dbusiface import NOT_SYNCHED_PATH
 from magicicada.helpers import humanize_bytes, get_data_file, get_builder, \
     log, NO_OP
 
@@ -47,16 +48,11 @@ META_QUEUE = 'meta'
 UBUNTU_ONE_ROOT = os.path.expanduser('~/Ubuntu One')
 
 # set up the logging for all the project
+# Instance of 'RootLogger' has no 'set_up' member
+# pylint: disable=E1103
 logger_helper.set_up()
 logger = logging.getLogger('magicicada.ui')
 
-DEBUG = os.getenv('DEBUG')
-if DEBUG:
-    console = logging.StreamHandler()
-    console.setLevel(logging.DEBUG)
-    logger.addHandler(console)
-
-
 # Instance of 'A' has no 'y' member
 # pylint: disable=E1101
 
@@ -81,7 +77,7 @@ class MagicicadaUI(object):
         self.builder = get_builder('gui.glade')
         self.builder.connect_signals(self)
 
-        if launchpad_available:
+        if LAUNCHPAD_AVAILABLE:
             # for more information about LaunchpadIntegration:
             # wiki.ubuntu.com/UbuntuDevelopment/Internationalisation/Coding
             helpmenu = self.builder.get_object('helpMenu')
@@ -105,6 +101,8 @@ class MagicicadaUI(object):
             'shares_to_others', 'shares_to_others_dialog',  # shares_to_others
             'shares_to_others_view', 'shares_to_others_store',
             'shares_to_others_close', 'metadata',  # metadata
+            'public_files', 'public_files_dialog',  # public_files
+            'public_files_view', 'public_files_store', 'public_files_close',
             'is_started', 'is_connected', 'is_online',  # status bar images
             'status_label', 'status_icon',  # status label and systray icon
             'metaq_view', 'contentq_view',  # queues tree views
@@ -122,18 +120,18 @@ class MagicicadaUI(object):
         self._make_view_sortable('folders')
         self._make_view_sortable('shares_to_me')
         self._make_view_sortable('shares_to_others')
+        self._make_view_sortable('public_files')
 
         self.metadata_dialogs = {}
         self.volumes = (self.folders, self.shares_to_me, self.shares_to_others)
-        self.windows = (self.main_window, self.about_dialog,
-                        self.folders_dialog, self.shares_to_me_dialog,
-                        self.shares_to_others_dialog)
 
-        icon_filename = get_data_file('media', 'logo-016.png')
-        self._icon = gtk.gdk.pixbuf_new_from_file(icon_filename)
-        self.status_icon.set_from_file(icon_filename)
-        for w in self.windows:
-            w.set_icon(self._icon)
+        self._icons = {}
+        for size in (16, 32, 48, 64, 128):
+            icon_filename = get_data_file('media', 'logo-%.3i.png' % size)
+            self._icons[size] = gtk.gdk.pixbuf_new_from_file(icon_filename)
+        self.status_icon.set_from_pixbuf(self._icons[16])
+        self.main_window.set_icon_list(*self._icons.values())
+        gtk.window_set_default_icon_list(*self._icons.values())
 
         about_fname = get_data_file('media', 'logo-128.png')
         self.about_dialog.set_logo(gtk.gdk.pixbuf_new_from_file(about_fname))
@@ -149,6 +147,9 @@ class MagicicadaUI(object):
         self.sd.content_queue_changed_callback = self.on_content_queue_changed
         self.sd.meta_queue_changed_callback = self.on_meta_queue_changed
         self.sd.on_metadata_ready_callback = self.on_metadata_ready
+        self.sd.on_initial_data_ready_callback = self.on_initial_data_ready
+        self.sd.on_initial_online_data_ready_callback = \
+            self.on_initial_online_data_ready
 
         self.widget_is_visible = lambda w: w.get_property('visible')
         self.widget_enabled = lambda w: self.widget_is_visible(w) and \
@@ -160,6 +161,8 @@ class MagicicadaUI(object):
         store = getattr(self, '%s_store' % view_name)
         view = getattr(self, '%s_view' % view_name)
         self._sorting_order[store] = {}
+        # this enforces that the order that columns will be shown and sorted
+        # matches the order in the underlying model
         for i, col in enumerate(view.get_columns()):
             col.set_clickable(True)
             col.connect('clicked', self.on_store_sort_column_changed, i, store)
@@ -175,7 +178,6 @@ class MagicicadaUI(object):
         dialog.set_border_width(10)
         # gtk.WIN_POS_CENTER makes dialogs overlap
         dialog.set_position(gtk.WIN_POS_MOUSE)
-        dialog.set_icon(self._icon)
 
         close_button = dialog.action_area.get_children()[-1]
         close_button.connect('clicked', self.on_metadata_close_clicked, path)
@@ -222,9 +224,10 @@ class MagicicadaUI(object):
 
     def on_stop_clicked(self, widget, data=None):
         """Stop syncdaemon."""
-        for v in self.volumes:
-            v.set_sensitive(False)
+        for volume in self.volumes:
+            volume.set_sensitive(False)
         self.metadata.set_sensitive(False)
+        self.public_files.set_sensitive(False)
 
         if self.widget_enabled(self.disconnect):
             self.on_disconnect_clicked(self.disconnect)
@@ -331,6 +334,24 @@ class MagicicadaUI(object):
         dialog.spinner.show()
         dialog.show()
 
+    def on_public_files_close_clicked(self, widget, data=None):
+        """Close the public_files dialog."""
+        self.public_files_dialog.response(gtk.RESPONSE_CLOSE)
+
+    def on_public_files_clicked(self, widget, data=None):
+        """Show information about public files."""
+        items = self.sd.public_files
+        if items is None:
+            items = {}
+
+        self.public_files_store.clear()
+        for item in items.itervalues():
+            row = (item.path, item.public_url, item.volume, item.node)
+            self.public_files_store.append(row)
+
+        self.public_files_dialog.run()
+        self.public_files_dialog.hide()
+
     def on_status_icon_activate(self, widget, data=None):
         """Systray icon was clicked."""
         if self.widget_is_visible(self.main_window):
@@ -370,10 +391,6 @@ class MagicicadaUI(object):
         self._activate_indicator(self.is_started)
         self.connect.set_sensitive(True)
 
-        for v in self.volumes:
-            v.set_sensitive(True)
-        self.metadata.set_sensitive(True)
-
         self._update_queues_and_status(self.sd.current_state)
 
     @log(logger)
@@ -482,13 +499,25 @@ class MagicicadaUI(object):
         dialog = self.metadata_dialogs[path]
         dialog.spinner.hide()
         dialog.view.show()
-        if metadata == dbusiface.NOT_SYNCHED_PATH:
+        if metadata == NOT_SYNCHED_PATH:
             # Metadata path doesn't exsit for syncdaemon
-            text = dbusiface.NOT_SYNCHED_PATH
+            text = NOT_SYNCHED_PATH
         else:
             text = '\n'.join('%s: %s' % i for i in metadata.iteritems())
         dialog.view.get_buffer().set_text(text)
 
+    @log(logger, level=logging.INFO)
+    def on_initial_data_ready(self):
+        """Initial data is now available in syncdaemon."""
+        for volume in self.volumes:
+            volume.set_sensitive(True)
+        self.metadata.set_sensitive(True)
+
+    @log(logger, level=logging.INFO)
+    def on_initial_online_data_ready(self):
+        """Online initial data is now available in syncdaemon."""
+        self.public_files.set_sensitive(True)
+
     # custom
 
     def _start_loading(self, what):
