@@ -27,7 +27,17 @@ from dbus import SessionBus
 from dbus.mainloop.glib import DBusGMainLoop
 from twisted.internet import defer
 
-from ubuntuone.syncdaemon.tools import SyncDaemonTool
+try:
+    # yes, they can be imported! pylint: disable=F0401,E0611
+    from ubuntuone.platform.tools import SyncDaemonTool, DBusClient
+    from ubuntuone.platform.dbus_interface import DBUS_IFACE_PUBLIC_FILES_NAME
+except ImportError:
+    # support for old structure (pre-Natty)
+    # yes, they can be imported! pylint: disable=F0401,E0611
+    from ubuntuone.syncdaemon.tools import SyncDaemonTool, DBusClient
+    from ubuntuone.syncdaemon.dbus_interface import (
+        DBUS_IFACE_PUBLIC_FILES_NAME,
+    )
 
 # log!
 logger = logging.getLogger('magicicada.dbusiface')
@@ -40,6 +50,8 @@ FolderData = collections.namedtuple('FolderData',
 ShareData = collections.namedtuple('ShareData', 'accepted access_level '
                                    'free_bytes name node_id other_username '
                                    'other_visible_name path volume_id')
+PublicFilesData = collections.namedtuple('PublicFilesData',
+                                         'volume node path public_url')
 
 # regular expressions for parsing MetaQueue data
 RE_OP_LISTDIR = re.compile("(ListDir)\(share_id=(.*?), node_id=(.*?), .*")
@@ -61,6 +73,15 @@ DBUSERR_PYKEYERROR = 'org.freedesktop.DBus.Python.KeyError'
 NOT_SYNCHED_PATH = "Not a valid path!"
 
 
+# some exceptions
+class ShareOperationError(Exception):
+    """Error on an operation on a share."""
+    def __init__(self, share_id, error):
+        self.share_id = share_id
+        self.error = error
+        Exception.__init__(self)
+
+
 def _is_retry_exception(err):
     """Check if the exception is a retry one."""
     if isinstance(err, dbus.exceptions.DBusException):
@@ -97,6 +118,7 @@ class DBusInterface(object):
         # magicicada's syncdaemon
         self.msd = msd
         logger.info("DBus interface starting")
+        self._public_files_deferred = None
 
         # set up dbus and related stuff
         loop = DBusGMainLoop(set_as_default=True)
@@ -116,6 +138,9 @@ class DBusInterface(object):
             (self._on_share_created, 'Shares', 'ShareCreated'),
             (self._on_share_deleted, 'Shares', 'ShareDeleted'),
             (self._on_share_changed, 'Shares', 'ShareChanged'),
+            (self._on_public_files_list, 'PublicFiles', 'PublicFilesList'),
+            (self._on_public_files_changed, 'PublicFiles',
+                                            'PublicAccessChanged'),
         ]
         self._dbus_matches = []
         for method, dbus_lastname, signal_name in _signals:
@@ -164,7 +189,7 @@ class DBusInterface(object):
         data = self._process_status(state)
         self.msd.on_sd_status_changed(*data)
 
-    def _on_content_queue_changed(self, _):
+    def _on_content_queue_changed(self, _=None):
         """Call the SD callback."""
         logger.info("Received Content Queue changed")
         self.msd.on_sd_content_queue_changed()
@@ -223,6 +248,57 @@ class DBusInterface(object):
         logger.info("Received Share changed")
         self.msd.on_sd_shares_changed()
 
+    def _on_public_files_changed(self, data):
+        """Call the SD callback."""
+        logger.debug("Received Public Files changed: %s", data)
+        pf = PublicFilesData(volume=data['share_id'], node=data['node_id'],
+                             path=data['path'], public_url=data['public_url'])
+        is_public = bool(data['is_public'])
+        self.msd.on_sd_public_files_changed(pf, is_public)
+
+    def _on_public_files_list(self, data):
+        """Call the SD callback."""
+        logger.info("Received Public Files list (%d)", len(data))
+        processed = []
+        for d in data:
+            logger.debug("    Public Files data: %s", d)
+            p = PublicFilesData(volume=d['volume_id'], node=d['node_id'],
+                                path=d['path'], public_url=d['public_url'])
+            processed.append(p)
+
+        # trigger the deferred if have one, else call the msd with the data
+        if self._public_files_deferred is None:
+            self.msd.on_sd_public_files_list(processed)
+        else:
+            d = self._public_files_deferred
+            self._public_files_deferred = None
+            d.callback(processed)
+
+    def get_public_files(self):
+        """Ask the Public Files info to syncdaemon."""
+        client = DBusClient(self._bus, '/publicfiles',
+                            DBUS_IFACE_PUBLIC_FILES_NAME)
+
+        # note that these callbacks do not come with the requested info, the
+        # method just will return None, and the real info will come later
+        # in a signal
+
+        def call_done(result):
+            """Call was succesful."""
+            logger.debug("Public files asked ok.")
+
+        def call_error(error):
+            """Call was not succesful."""
+            logger.error("Public files asked with error: %s", error)
+
+        client.call_method('get_public_files',
+                           reply_handler=call_done, error_handler=call_error)
+
+        # return a stored deferred that will be triggered later with the signal
+        d = defer.Deferred()
+        self._public_files_deferred = d
+        return d
+
     @retryable
     def get_content_queue(self):
         """Get the content queue from SDT."""
@@ -293,7 +369,7 @@ class DBusInterface(object):
         """Use MetaQueue dictionary and prepare command data."""
         if op in ('AccountInquiry', 'FreeSpaceInquiry', 'Query', 'ListShares',
                   'GetPublicFiles', 'ListVolumes', 'ChangePublicAccess',
-                  'AnswerShare'):
+                  'AnswerShare', 'GetDelta'):
             return QueueData(operation=op, path=None, node=None, share=None)
 
         if op in ('ListDir', 'Unlink'):
@@ -473,3 +549,19 @@ class DBusInterface(object):
         d = self.sync_daemon_tool.get_metadata(path)
         d.addCallbacks(process, fix_failure)
         return d
+
+    @retryable
+    @defer.inlineCallbacks
+    def accept_share(self, share_id):
+        """Accept a share."""
+        logger.debug("Accept share %s started", share_id)
+        try:
+            result = yield self.sync_daemon_tool.accept_share(share_id)
+        except dbus.exceptions.DBusException, e:
+            error = str(e.args[0])
+            logger.debug("Accept share %s crashed: %s", share_id, error)
+            raise ShareOperationError(share_id=share_id, error=error)
+
+        logger.debug("Accept share %s finished: %s", share_id, result)
+        if 'error' in result:
+            raise ShareOperationError(share_id=share_id, error=result['error'])
