@@ -27,11 +27,38 @@ from twisted.internet import defer
 from magicicada import dbusiface
 from magicicada.tests.helpers import MementoHandler
 
+try:
+    # yes, they can be imported! pylint: disable=F0401,E0611
+    from ubuntuone.platform.dbus_interface import DBUS_IFACE_PUBLIC_FILES_NAME
+except ImportError:
+    # support for old structure (pre-Natty)
+    # yes, they can be imported! pylint: disable=F0401,E0611
+    from ubuntuone.syncdaemon.dbus_interface import (
+        DBUS_IFACE_PUBLIC_FILES_NAME,
+    )
+
 
 # It's ok to access private data in the test suite
 # pylint: disable=W0212
 
 
+class FakeDBusClient(object):
+    """Fake DBusClient, but with different behaviour at instantiation time."""
+
+    def __init__(self, *args):
+        self.init_args = None
+        self.method_call_args = None
+
+    def call_method(self, *args, **kwargs):
+        """Record the arguments of the method call."""
+        self.method_call_args = args, kwargs
+
+    def __call__(self, *args):
+        """Simulate instantiation."""
+        self.init_args = args
+        return self
+
+
 class FakeSessionBus(object):
     """Fake Session Bus."""
 
@@ -59,6 +86,9 @@ class FakeSessionBus(object):
         else:
             raise self.fake_name_owner
 
+    def send_message_with_reply(self, *a, **k):
+        """Fake."""
+
 
 class CallLoguer(object):
     """Class that logs the methods called."""
@@ -104,6 +134,12 @@ class SafeTests(TwistedTestCase):
 
     def setUp(self):
         """Set up."""
+        self.handler = MementoHandler()
+        self.handler.setLevel(logging.DEBUG)
+        logger = logging.getLogger('magicicada.dbusiface')
+        logger.addHandler(self.handler)
+        logger.setLevel(logging.DEBUG)
+
         dbusiface.SessionBus = FakeSessionBus
         dbusiface.SyncDaemonTool = FakeSDTool
         self.fsd = FakeSyncDaemon()
@@ -199,6 +235,17 @@ class TestSignalHooking(SafeTests):
         self.assertEqual(self._get_hooked('Shares', 'ShareChanged'),
                          self.dbus._on_share_changed)
 
+    def test_public_files_list(self):
+        """Test public files list callback."""
+        self.assertEqual(self._get_hooked('PublicFiles', 'PublicFilesList'),
+                         self.dbus._on_public_files_list)
+
+    def test_public_files_changed(self):
+        """Test public files changed callback."""
+        self.assertEqual(self._get_hooked('PublicFiles',
+                         'PublicAccessChanged'),
+                         self.dbus._on_public_files_changed)
+
 
 class TestSimpleCalls(SafeTests):
     """Tests for some simple calls."""
@@ -263,13 +310,13 @@ class TestDataProcessingNameOwner(SafeTests):
         self.dbus._on_name_owner_changed("foo", "bar", "baz")
         self.get_msd_called(None)
 
-    def test_name_owner_changed_yes_syncdaemon_TF(self):
+    def test_name_owner_changed_yes_syncdaemon_true_false(self):
         """Test name owner changed callback."""
         self.dbus._on_name_owner_changed("com.ubuntuone.SyncDaemon", "T", "")
         rcv, = self.get_msd_called("on_sd_name_owner_changed")
         self.assertEqual(rcv, False)
 
-    def test_name_owner_changed_yes_syncdaemon_FT(self):
+    def test_name_owner_changed_yes_syncdaemon_false_true(self):
         """Test name owner changed callback."""
         self.dbus._on_name_owner_changed("com.ubuntuone.SyncDaemon", "", "T")
         rcv, = self.get_msd_called("on_sd_name_owner_changed")
@@ -279,11 +326,16 @@ class TestDataProcessingNameOwner(SafeTests):
 class TestDataProcessingCQ(SafeTests):
     """Process CQ data before sending it to SyncDaemon."""
 
-    def test_content_queue_changed_signal(self):
-        """Test content queue changed signal."""
+    def test_content_queue_changed_signal_withsomething(self):
+        """Test content queue changed signal, old version with data."""
         self.dbus._on_content_queue_changed(None)
         self.get_msd_called("on_sd_content_queue_changed")
 
+    def test_content_queue_changed_signal_nodata(self):
+        """Test content queue changed signal new version (just the signal)."""
+        self.dbus._on_content_queue_changed()
+        self.get_msd_called("on_sd_content_queue_changed")
+
     @defer.inlineCallbacks
     def test_nodata(self):
         """Test with no data in the queue."""
@@ -371,7 +423,7 @@ class TestDataProcessingMQ(SafeTests):
         self.assertEqual(data.node, None)
 
     @defer.inlineCallbacks
-    def test_GetPublicFiles_old(self):
+    def test_getpublicfiles_old(self):
         """Test meta with GetPublicFiles."""
         cmd = 'GetPublicFiles'
         self.fake_sdt_response('waiting_metadata', [cmd])
@@ -383,7 +435,7 @@ class TestDataProcessingMQ(SafeTests):
         self.assertEqual(data.node, None)
 
     @defer.inlineCallbacks
-    def test_AccountInquiry_old(self):
+    def test_accountinquiry_old(self):
         """Test meta with AccountInquiry."""
         cmd = 'AccountInquiry'
         self.fake_sdt_response('waiting_metadata', [cmd])
@@ -395,7 +447,7 @@ class TestDataProcessingMQ(SafeTests):
         self.assertEqual(data.node, None)
 
     @defer.inlineCallbacks
-    def test_FreeSpaceInquiry_old(self):
+    def test_freespaceinquiry_old(self):
         """Test meta with FreeSpaceInquiry."""
         cmd = 'FreeSpaceInquiry'
         self.fake_sdt_response('waiting_metadata', [cmd])
@@ -407,7 +459,7 @@ class TestDataProcessingMQ(SafeTests):
         self.assertEqual(data.node, None)
 
     @defer.inlineCallbacks
-    def test_ListShares_old(self):
+    def test_listshares_old(self):
         """Test meta with ListShares."""
         cmd = 'ListShares'
         self.fake_sdt_response('waiting_metadata', [cmd])
@@ -419,7 +471,7 @@ class TestDataProcessingMQ(SafeTests):
         self.assertEqual(data.node, None)
 
     @defer.inlineCallbacks
-    def test_ListVolumes_old(self):
+    def test_listvolumes_old(self):
         """Test meta with ListVolumes."""
         cmd = 'ListVolumes'
         self.fake_sdt_response('waiting_metadata', [cmd])
@@ -431,7 +483,7 @@ class TestDataProcessingMQ(SafeTests):
         self.assertEqual(data.node, None)
 
     @defer.inlineCallbacks
-    def test_Query_old(self):
+    def test_query_old(self):
         """Test meta with Query."""
         cmd = 'Query'
         self.fake_sdt_response('waiting_metadata', [cmd])
@@ -443,7 +495,7 @@ class TestDataProcessingMQ(SafeTests):
         self.assertEqual(data.node, None)
 
     @defer.inlineCallbacks
-    def test_ListDir_old(self):
+    def test_listdir_old(self):
         """Test meta with ListDir."""
         cmd = 'ListDir(share_id=a, node_id=b, server_hash=c)'
         self.fake_sdt_response('waiting_metadata', [cmd])
@@ -455,7 +507,7 @@ class TestDataProcessingMQ(SafeTests):
         self.assertEqual(data.node, 'b')
 
     @defer.inlineCallbacks
-    def test_MakeDir_old(self):
+    def test_makedir_old(self):
         """Test meta with MakeDir."""
         cmd = 'MakeDir(share_id=a, parent_id=b, name=c, marker=d)'
         self.fake_sdt_response('waiting_metadata', [cmd])
@@ -467,7 +519,7 @@ class TestDataProcessingMQ(SafeTests):
         self.assertEqual(data.node, None)
 
     @defer.inlineCallbacks
-    def test_MakeFile_old(self):
+    def test_makefile_old(self):
         """Test meta with MakeFile."""
         cmd = 'MakeFile(share_id=a, parent_id=b, name=c, marker=d)'
         self.fake_sdt_response('waiting_metadata', [cmd])
@@ -479,7 +531,7 @@ class TestDataProcessingMQ(SafeTests):
         self.assertEqual(data.node, None)
 
     @defer.inlineCallbacks
-    def test_Unlink_old(self):
+    def test_unlink_old(self):
         """Test meta with Unlink."""
         cmd = 'Unlink(share_id=a, node_id=b, server_hash=c)'
         self.fake_sdt_response('waiting_metadata', [cmd])
@@ -491,7 +543,7 @@ class TestDataProcessingMQ(SafeTests):
         self.assertEqual(data.node, 'b')
 
     @defer.inlineCallbacks
-    def test_Move_old(self):
+    def test_move_old(self):
         """Test meta with Move."""
         cmd = 'Move(share_id=a, node_id=b, old_parent_id=c, '\
               'new_parent_id=d, new_name=e)'
@@ -504,7 +556,7 @@ class TestDataProcessingMQ(SafeTests):
         self.assertEqual(data.node, 'b')
 
     @defer.inlineCallbacks
-    def test_ChangePublicAccess_old(self):
+    def test_changepublicaccess_old(self):
         """Test meta with ChangePublicAccess."""
         cmd = 'ChangePublicAccess'
         self.fake_sdt_response('waiting_metadata', [cmd])
@@ -516,7 +568,7 @@ class TestDataProcessingMQ(SafeTests):
         self.assertEqual(data.node, None)
 
     @defer.inlineCallbacks
-    def test_AnswerShare_old(self):
+    def test_answershare_old(self):
         """Test meta with AnswerShare."""
         cmd = 'AnswerShare'
         self.fake_sdt_response('waiting_metadata', [cmd])
@@ -528,7 +580,7 @@ class TestDataProcessingMQ(SafeTests):
         self.assertEqual(data.node, None)
 
     @defer.inlineCallbacks
-    def test_GetPublicFiles_dict(self):
+    def test_getpublicfiles_dict(self):
         """Test meta with GetPublicFiles."""
         cmd = ('GetPublicFiles', {})
         self.fake_sdt_response('waiting_metadata', [cmd])
@@ -540,7 +592,7 @@ class TestDataProcessingMQ(SafeTests):
         self.assertEqual(data.node, None)
 
     @defer.inlineCallbacks
-    def test_AccountInquiry_dict(self):
+    def test_accountinquiry_dict(self):
         """Test meta with AccountInquiry."""
         cmd = ('AccountInquiry', {})
         self.fake_sdt_response('waiting_metadata', [cmd])
@@ -552,7 +604,7 @@ class TestDataProcessingMQ(SafeTests):
         self.assertEqual(data.node, None)
 
     @defer.inlineCallbacks
-    def test_FreeSpaceInquiry_dict(self):
+    def test_freespaceinquiry_dict(self):
         """Test meta with FreeSpaceInquiry."""
         cmd = ('FreeSpaceInquiry', {})
         self.fake_sdt_response('waiting_metadata', [cmd])
@@ -564,7 +616,7 @@ class TestDataProcessingMQ(SafeTests):
         self.assertEqual(data.node, None)
 
     @defer.inlineCallbacks
-    def test_ListShares_dict(self):
+    def test_listshares_dict(self):
         """Test meta with ListShares."""
         cmd = ('ListShares', {})
         self.fake_sdt_response('waiting_metadata', [cmd])
@@ -576,7 +628,7 @@ class TestDataProcessingMQ(SafeTests):
         self.assertEqual(data.node, None)
 
     @defer.inlineCallbacks
-    def test_ListVolumes_dict(self):
+    def test_listvolumes_dict(self):
         """Test meta with ListVolumes."""
         cmd = ('ListVolumes', {})
         self.fake_sdt_response('waiting_metadata', [cmd])
@@ -588,7 +640,7 @@ class TestDataProcessingMQ(SafeTests):
         self.assertEqual(data.node, None)
 
     @defer.inlineCallbacks
-    def test_Query_dict(self):
+    def test_query_dict(self):
         """Test meta with Query."""
         cmd = ('Query', {})
         self.fake_sdt_response('waiting_metadata', [cmd])
@@ -600,7 +652,7 @@ class TestDataProcessingMQ(SafeTests):
         self.assertEqual(data.node, None)
 
     @defer.inlineCallbacks
-    def test_ListDir_dict(self):
+    def test_listdir_dict(self):
         """Test meta with ListDir."""
         cmd = ('ListDir', dict(share_id='a', node_id='b',
                                server_hash='c', path='d'))
@@ -613,7 +665,7 @@ class TestDataProcessingMQ(SafeTests):
         self.assertEqual(data.node, 'b')
 
     @defer.inlineCallbacks
-    def test_MakeDir_dict(self):
+    def test_makedir_dict(self):
         """Test meta with MakeDir."""
         cmd = ('MakeDir', dict(share_id='a', parent_id='b',
                                name='c', marker='d'))
@@ -626,7 +678,7 @@ class TestDataProcessingMQ(SafeTests):
         self.assertEqual(data.node, None)
 
     @defer.inlineCallbacks
-    def test_MakeFile_dict(self):
+    def test_makefile_dict(self):
         """Test meta with MakeFile."""
         cmd = ('MakeFile', dict(share_id='a', parent_id='b',
                                 name='c', marker='d'))
@@ -639,7 +691,7 @@ class TestDataProcessingMQ(SafeTests):
         self.assertEqual(data.node, None)
 
     @defer.inlineCallbacks
-    def test_Unlink_dict(self):
+    def test_unlink_dict(self):
         """Test meta with Unlink."""
         cmd = ('Unlink', dict(share_id='a', node_id='b',
                               server_hash='c', path='d'))
@@ -652,7 +704,7 @@ class TestDataProcessingMQ(SafeTests):
         self.assertEqual(data.node, 'b')
 
     @defer.inlineCallbacks
-    def test_Move_dict(self):
+    def test_move_dict(self):
         """Test meta with Move."""
         cmd = ('Move', dict(share_id='a', node_id='b', old_parent_id='c',
                             new_parent_id='d', new_name='e', path='f'))
@@ -665,7 +717,7 @@ class TestDataProcessingMQ(SafeTests):
         self.assertEqual(data.node, 'b')
 
     @defer.inlineCallbacks
-    def test_ChangePublicAccess_dict(self):
+    def test_changepublicaccess_dict(self):
         """Test meta with ChangePublicAccess."""
         cmd = ('ChangePublicAccess', {})
         self.fake_sdt_response('waiting_metadata', [cmd])
@@ -677,7 +729,7 @@ class TestDataProcessingMQ(SafeTests):
         self.assertEqual(data.node, None)
 
     @defer.inlineCallbacks
-    def test_AnswerShare_dict(self):
+    def test_answershare_dict(self):
         """Test meta with AnswerShare."""
         cmd = ('AnswerShare', {})
         self.fake_sdt_response('waiting_metadata', [cmd])
@@ -688,6 +740,18 @@ class TestDataProcessingMQ(SafeTests):
         self.assertEqual(data.share, None)
         self.assertEqual(data.node, None)
 
+    @defer.inlineCallbacks
+    def test_getdelta(self):
+        """Test meta with AnswerShare."""
+        cmd = ('GetDelta', {})
+        self.fake_sdt_response('waiting_metadata', [cmd])
+        rcv = yield self.dbus.get_meta_queue()
+        data = rcv[0]
+        self.assertEqual(data.operation, 'GetDelta')
+        self.assertEqual(data.path, None)
+        self.assertEqual(data.share, None)
+        self.assertEqual(data.node, None)
+
 
 class TestDataProcessingFolders(SafeTests):
     """Process Folders data before sending it to SyncDaemon."""
@@ -910,6 +974,138 @@ class TestDataProcessingShares(SafeTests):
         self.assertEqual(share.volume_id, 'vol')
 
 
+class TestPublicFiles(SafeTests):
+    """PublicFiles data handling and related services."""
+
+    def test_public_files_changed_yes(self):
+        """Call the changed callback."""
+        d = dict(share_id='share', node_id='node', is_public='True',
+                 public_url='url', path='path')
+        self.dbus._on_public_files_changed(d)
+        pf, is_public = self.get_msd_called("on_sd_public_files_changed")
+        self.assertEqual(pf.volume, 'share')
+        self.assertEqual(pf.node, 'node')
+        self.assertEqual(pf.public_url, 'url')
+        self.assertEqual(pf.path, 'path')
+        self.assertEqual(is_public, True)
+
+    def test_public_files_changed_no(self):
+        """Call the changed callback."""
+        d = dict(share_id='share', node_id='node', is_public='',
+                 public_url='url', path='path')
+        self.dbus._on_public_files_changed(d)
+        pf, is_public = self.get_msd_called("on_sd_public_files_changed")
+        self.assertEqual(pf.volume, 'share')
+        self.assertEqual(pf.node, 'node')
+        self.assertEqual(pf.public_url, 'url')
+        self.assertEqual(pf.path, 'path')
+        self.assertEqual(is_public, False)
+
+    def test_public_files_empty(self):
+        """Call the callback without info."""
+        self.dbus._on_public_files_list([])
+        res, = self.get_msd_called("on_sd_public_files_list")
+        self.assertEqual(res, [])
+
+    def test_public_files_one(self):
+        """Call the callback with a public file."""
+        d = dict(volume_id='volume', node_id='node',
+                 public_url='url', path='path')
+        self.dbus._on_public_files_list([d])
+        res, = self.get_msd_called("on_sd_public_files_list")
+        self.assertEqual(len(res), 1)
+        pf = res[0]
+        self.assertEqual(pf.volume, 'volume')
+        self.assertEqual(pf.node, 'node')
+        self.assertEqual(pf.public_url, 'url')
+        self.assertEqual(pf.path, 'path')
+
+    def test_public_files_more(self):
+        """Call the callback with a mixed content."""
+        d1 = dict(volume_id='volume1', node_id='node1',
+                  public_url='url1', path='path1')
+        d2 = dict(volume_id='volume2', node_id='node2',
+                  public_url='url2', path='path2')
+        d3 = dict(volume_id='volume3', node_id='node3',
+                  public_url='url3', path='path3')
+        self.dbus._on_public_files_list([d1, d2, d3])
+        res, = self.get_msd_called("on_sd_public_files_list")
+        self.assertEqual(len(res), 3)
+        pf = res[0]
+        self.assertEqual(pf.volume, 'volume1')
+        self.assertEqual(pf.node, 'node1')
+        self.assertEqual(pf.public_url, 'url1')
+        self.assertEqual(pf.path, 'path1')
+        pf = res[1]
+        self.assertEqual(pf.volume, 'volume2')
+        self.assertEqual(pf.node, 'node2')
+        self.assertEqual(pf.public_url, 'url2')
+        self.assertEqual(pf.path, 'path2')
+        pf = res[2]
+        self.assertEqual(pf.volume, 'volume3')
+        self.assertEqual(pf.node, 'node3')
+        self.assertEqual(pf.public_url, 'url3')
+        self.assertEqual(pf.path, 'path3')
+
+    def test_getpublicfiles_deferred(self):
+        """The method should return a deferred and store it."""
+        d = self.dbus.get_public_files()
+        self.assertTrue(isinstance(d, defer.Deferred))
+        self.assertIdentical(d, self.dbus._public_files_deferred)
+
+    @defer.inlineCallbacks
+    def test_public_files_when_asked(self):
+        """If we receive the signal and data was asked, change behaviour."""
+        # set up stuff
+        d = defer.Deferred()
+        self.dbus._public_files_deferred = d
+
+        # call the signal
+        self.dbus._on_public_files_list([])
+
+        # check that the deferred was callbacked, and not the normal SD method
+        result = yield d
+        self.assertEqual(result, [])
+        self.assertTrue(self.fsd._called_method[0] is None)
+
+    def test_getpublicfiles_asktoclient_ok(self):
+        """The info was requested to the DBusClient, and was ok."""
+        # inject a different DBusClient
+        fake_dbusclient = FakeDBusClient()
+        self.patch(dbusiface, 'DBusClient', fake_dbusclient)
+
+        # call, and check init and method args
+        self.dbus.get_public_files()
+        self.assertEqual(fake_dbusclient.init_args, (self.dbus._bus,
+                                '/publicfiles', DBUS_IFACE_PUBLIC_FILES_NAME))
+        (method_name,), kwargs = fake_dbusclient.method_call_args
+        done_cback = kwargs['reply_handler']
+        self.assertEqual(method_name, 'get_public_files')
+
+        # trigger done callback and check logging
+        done_cback(None)
+        self.assertTrue(self.handler.check_debug("Public files asked ok."))
+
+    def test_getpublicfiles_asktoclient_error(self):
+        """The info was requested to the DBusClient, and was not ok."""
+        # inject a different DBusClient
+        fake_dbusclient = FakeDBusClient()
+        self.patch(dbusiface, 'DBusClient', fake_dbusclient)
+
+        # call, and check init and method args
+        self.dbus.get_public_files()
+        self.assertEqual(fake_dbusclient.init_args, (self.dbus._bus,
+                                '/publicfiles', DBUS_IFACE_PUBLIC_FILES_NAME))
+        (method_name,), kwargs = fake_dbusclient.method_call_args
+        error_cback = kwargs['error_handler']
+        self.assertEqual(method_name, 'get_public_files')
+
+        # trigger error callback and check logging
+        error_cback('foo')
+        self.assertTrue(self.handler.check_error(
+                        "Public files asked with error: foo"))
+
+
 class TestToolActions(SafeTests):
     """Actions against SD.tools.
 
@@ -944,8 +1140,10 @@ class TestLogs(SafeTests):
     def setUp(self):
         """Set up."""
         self.handler = MementoHandler()
-        logging.getLogger('magicicada.dbusiface').addHandler(self.handler)
         self.handler.setLevel(logging.DEBUG)
+        logger = logging.getLogger('magicicada.dbusiface')
+        logger.addHandler(self.handler)
+        logger.setLevel(logging.DEBUG)
         SafeTests.setUp(self)
 
     def test_instancing(self):
@@ -1052,13 +1250,13 @@ class TestLogs(SafeTests):
         self.assertTrue(self.handler.check_info("Received Name Owner changed"))
         self.assertTrue(self.handler.check_debug("Name Owner data: u'' u'T'"))
 
-    def test_name_owner_changed_yes_syncdaemon_TF(self):
+    def test_name_owner_changed_yes_syncdaemon_true_false(self):
         """Test name owner changed callback, SD value bad."""
         self.dbus._on_name_owner_changed("com.ubuntuone.SyncDaemon", "F", "T")
         self.assertTrue(self.handler.check_info("Received Name Owner changed"))
         self.assertTrue(self.handler.check_debug("Name Owner data: u'F' u'T'"))
-        self.assertTrue(self.handler.check("ERROR",
-                        "Name Owner invalid data: Same bool in old and new!"))
+        msg = "Name Owner invalid data: Same bool in old and new!"
+        self.assertTrue(self.handler.check_error(msg))
 
     def test_folder_created_changed(self):
         """Test folder created changed callback."""
@@ -1091,11 +1289,31 @@ class TestLogs(SafeTests):
         self.dbus._on_share_deleted("foo")
         self.assertTrue(self.handler.check_info("Received Share deleted"))
 
-    def test_share__changed(self):
+    def test_share_changed(self):
         """Test share changed callback."""
         self.dbus._on_share_changed("foo")
         self.assertTrue(self.handler.check_info("Received Share changed"))
 
+    @defer.inlineCallbacks
+    def test_public_files_list(self):
+        """Test public files list callback."""
+        d = dict(volume_id='volume', node_id='node',
+                 public_url='url', path='path')
+        yield self.dbus._on_public_files_list([d])
+        self.assertTrue(self.handler.check_info(
+                        "Received Public Files list (1)"))
+        self.assertTrue(self.handler.check_debug(
+                        "    Public Files data: %s" % d))
+
+    @defer.inlineCallbacks
+    def test_public_files_changed(self):
+        """Test public files changed callback."""
+        d = dict(share_id='volume', node_id='node', is_public='',
+                 public_url='url', path='path')
+        yield self.dbus._on_public_files_changed(d)
+        self.assertTrue(self.handler.check_debug(
+                        "Received Public Files changed: %s" % d))
+
     @defer.inlineCallbacks
     def test_content_queue_processing(self):
         """Test with one item in the queue."""
@@ -1270,3 +1488,52 @@ class RetryDecoratorTests(TwistedTestCase):
         d.addCallbacks(lambda _: deferred.errback(Exception()),
                        lambda _: deferred.callback(True))
         return deferred
+
+
+class TestHandlingShares(SafeTests):
+    """Handle shares."""
+
+    @defer.inlineCallbacks
+    def test_accept_share_ok(self):
+        """Accepting share finishes ok."""
+        d = dict(volume_id='foo', answer='bar')
+        self.fake_sdt_response('accept_share', d)
+        yield self.dbus.accept_share('foo')
+        self.assertTrue(self.handler.check_debug(
+                        "Accept share foo started", "foo"))
+        self.assertTrue(self.handler.check_debug(
+                        "Accept share foo finished", str(d)))
+
+    @defer.inlineCallbacks
+    def test_accept_share_bad(self):
+        """Accepting share finishes bad."""
+        d = dict(volume_id='foo', answer='bar', error='baz')
+        self.fake_sdt_response('accept_share', d)
+        try:
+            yield self.dbus.accept_share('foo')
+        except dbusiface.ShareOperationError, e:
+            self.assertEqual(e.share_id, 'foo')
+            self.assertEqual(e.error, 'baz')
+        else:
+            raise Exception("Test should have raised an exception")
+        self.assertTrue(self.handler.check_debug(
+                        "Accept share foo started", "foo"))
+        self.assertTrue(self.handler.check_debug(
+                        "Accept share foo finished", str(d)))
+
+    @defer.inlineCallbacks
+    def test_accept_share_ugly_error(self):
+        """Accepting share went really bad."""
+        e = dbus.exceptions.DBusException('ugly!')
+        self.fake_sdt_response('accept_share', e)
+        try:
+            yield self.dbus.accept_share('foo')
+        except dbusiface.ShareOperationError, e:
+            self.assertEqual(e.share_id, 'foo')
+            self.assertEqual(e.error, "ugly!")
+        else:
+            raise Exception("Test should have raised an exception")
+        self.assertTrue(self.handler.check_debug(
+                        "Accept share foo started", "foo"))
+        self.assertTrue(self.handler.check_debug(
+                        "Accept share foo crashed", "ugly!"))
