@@ -32,9 +32,9 @@ import pango
 from twisted.trial.unittest import TestCase
 
 from magicicada import MagicicadaUI, CONTENT_QUEUE, META_QUEUE, \
-                       UBUNTU_ONE_ROOT, syncdaemon
+    NOT_SYNCHED_PATH, UBUNTU_ONE_ROOT, syncdaemon
 from magicicada.dbusiface import QueueData, FolderData, ShareData, \
-                                 NOT_SYNCHED_PATH
+    PublicFilesData
 from magicicada.helpers import NO_OP, humanize_bytes, get_data_file
 from magicicada.tests.helpers import MementoHandler
 
@@ -52,6 +52,25 @@ from magicicada.tests.helpers import MementoHandler
 # pylint: disable=E1103
 
 
+SAMPLE_PUBLIC_FILES = {
+    u'4d0ffa01-5375-4e67-bc47-d993b721d8af':
+    PublicFilesData(volume=u'',
+                    node=u'4d0ffa01-5375-4e67-bc47-d993b721d8af',
+                    path=u'/home/user/Ubuntu One/test.png',
+                    public_url=u'http://ubuntuone.com/p/CUG/'),
+    u'1be1ea29-426e-41b3-8853-5cce03b0c511':
+    PublicFilesData(volume=u'',
+                    node=u'1be1ea29-426e-41b3-8853-5cce03b0c511',
+                    path=u'/home/user/Ubuntu One/public/text.txt',
+                    public_url=u'http://ubuntuone.com/p/6TX/'),
+    u'8dba0484-86e9-4505-8fba-db3db7cc551d':
+    PublicFilesData(volume=u'f5cc6b0d-85a0-4046-a72f-bd42c41538cb',
+                    node=u'8dba0484-86e9-4505-8fba-db3db7cc551d',
+                    path=u'/home/user/udf0/yadda.py',
+                    public_url=u'http://ubuntuone.com/p/U4R/'),
+}
+
+
 def process_gtk_pendings():
     """Process all gtk pending events."""
     while gtk.events_pending():
@@ -101,8 +120,13 @@ class FakedSyncdaemon(object):
         self.content_queue_changed_callback = NO_OP
         self.meta_queue_changed_callback = NO_OP
         self.on_metadata_ready_callback = None  # mandatory
+        self.on_initial_data_ready_callback = NO_OP
+        self.on_initial_online_data_ready_callback = NO_OP
         self.shutdown = NO_OP
 
+        # Lambda may not be necessary
+        # pylint: disable=W0108
+
         self.start = lambda: setattr(self.current_state, 'is_started', True)
         self.quit = lambda: setattr(self.current_state, 'is_started', False)
         self.connect = lambda: setattr(self.current_state,
@@ -116,6 +140,7 @@ class MagicicadaUITestCase(TestCase):
     """UI test cases for Magicicada UI."""
 
     TEST_FILE = get_data_file('tests', 'metadata-test.txt')
+    data_type = mapping = store = view = None
 
     def setUp(self):
         """Init."""
@@ -135,30 +160,6 @@ class MagicicadaUITestCase(TestCase):
             # no, I'm not raising a bool. pylint: disable=E0702
             raise self._failed_test
 
-    def __getattribute__(self, name):
-        """Overwrite the assert methods with safer ones.
-
-        This way if a test called by gobject in the future fails, it
-        makes the whole test suite fail.
-        """
-        if name.startswith('assert') and hasattr(TestCase, name):
-
-            def proxy(*args, **kwargs):
-                """Function that will call the real assert."""
-                real_assert = getattr(TestCase, name)
-                try:
-                    real_assert(self, *args, **kwargs)
-                except Exception, e:
-                    self._failed_test = e
-                    raise
-            return proxy
-        else:
-            return TestCase.__getattribute__(self, name)
-
-        if self._failed_test:
-            # no, I'm not raising a bool. pylint: disable=E0702
-            raise self._failed_test
-
     def __getattribute__(self, name):
         """Overwrite the assert methods with safer ones.
 
@@ -190,41 +191,56 @@ class MagicicadaUITestCase(TestCase):
         self.ui.on_connect_clicked(self.ui.connect)
         self.ui.on_connected()
 
-    def build_some_data(self, data_type, limit=5):
+    def build_some_data(self, limit=5):
         """Build some data using named_tuple 'data_type'."""
-        attrs = data_type._fields
+        assert self.data_type is not None, 'class muct provide a data_type'
+
+        attrs = self.data_type._fields
         result = []
         for i in xrange(limit):
             kwargs = dict([(attr, '%s %i' % (attr, i)) for attr in attrs])
-            result.append(data_type(**kwargs))
+            # self.data_type is not callable, pylint: disable=E1102
+            result.append(self.data_type(**kwargs))
         return result
 
-    def assert_store_correct(self, store, items, mapping, markup=None):
-        """Test that 'store' has 'items' as content."""
+    def assert_store_correct(self, items, markup=None):
+        """Test that 'self.store' has 'items' as content."""
+        assert self.mapping is not None, 'class must provide a mapping'
+        assert self.store is not None, 'class must provide a store'
+
         msg = 'amount of rows for %s must be %s (got %s).'
-        self.assertEqual(len(store), len(items),
-                         msg % (store, len(items), len(store)))
+        self.assertEqual(len(self.store), len(items),
+                         msg % (self.store, len(items), len(self.store)))
 
         # assert rows content equal to items content
-        tree_iter = store.get_iter_root()
+        tree_iter = self.store.get_iter_root()
         tmp = list(reversed(items))
         do_markup = markup is not None
         msg = "column %i ('%s') must be '%s' (got '%s' instead)"
         while tree_iter is not None:
             head = tmp.pop()
-            for i, field in mapping:
-                actual, = store.get(tree_iter, i)
+            for i, field in self.mapping:
+                actual, = self.store.get(tree_iter, i)
                 expected = getattr(head, field)
-                if store.get_column_type(i).name == 'gboolean':
+                if self.store.get_column_type(i).name == 'gboolean':
                     expected = bool(expected)
                 elif do_markup:
                     expected = markup(expected)
                 self.assertEqual(expected, actual,
                                  msg % (i, field, expected, actual))
 
-            tree_iter = store.iter_next(tree_iter)
+            tree_iter = self.store.iter_next(tree_iter)
             do_markup = False  # only for first row
 
+    def debug_store(self):
+        """Print the whole content of a store."""
+        store_iter = self.store.get_iter_root()
+        columns = self.store.get_n_columns()
+        print '\nShowing contents of store:', self.store
+        while store_iter is not None:
+            print self.store.get(store_iter, *range(columns))
+            store_iter = self.store.iter_next(store_iter)
+
     def assert_indicator_disabled(self, indicator):
         """Test that 'indicator' is not sensitive."""
         self.assertFalse(indicator.is_sensitive(),
@@ -283,7 +299,41 @@ class MagicicadaUITestCase(TestCase):
         msg = '%s must not skip taskbar.'
         self.assertFalse(dialog.get_skip_taskbar_hint(), msg % dialog_name)
 
-        self.assertEqual(dialog.get_icon(), self.ui._icon)
+    def assert_sort_order_correct(self, column, idx, expected_order):
+        """Check that sort order is correctly set for 'self.store'."""
+        assert self.store is not None, 'class must provide a store'
+
+        msg0 = 'Store sort id must be %r (got %r instead).'
+        msg1 = 'Store sort order must be %r (got %r instead).'
+        msg3 = 'Column sort order must be %r (got %r instead).'
+
+        actual_id, actual_order = self.store.get_sort_column_id()
+
+        # store sort column id and order
+        self.assertEqual(idx, actual_id, msg0 % (idx, actual_id))
+        self.assertEqual(expected_order, actual_order,
+                         msg1 % (expected_order, actual_order))
+
+        # column sort order
+        actual_order = column.get_sort_order()
+        self.assertEqual(expected_order, actual_order,
+                         msg3 % (expected_order, actual_order))
+
+    def assert_sort_indicator_correct(self, column):
+        """Check that sort indicator is correctly set."""
+        assert self.view is not None, 'class must provide a view'
+
+        msg = 'Column %s must have sort indicator %s.'
+        colname = column.get_name()
+        # column sort indicator
+        self.assertTrue(column.get_sort_indicator(), msg % (colname, 'on'))
+
+        # all the other columns must not have the sort indicator on
+        for other_column in self.view.get_columns():
+            if other_column.get_name() == colname:
+                continue
+            self.assertFalse(other_column.get_sort_indicator(),
+                             msg % (other_column.get_name(), 'off'))
 
 
 class MagicicadaUIBasicTestCase(MagicicadaUITestCase):
@@ -305,10 +355,16 @@ class MagicicadaUIBasicTestCase(MagicicadaUITestCase):
         """UI can be created and main_window is visible."""
         self.assertTrue(self.ui.widget_is_visible(self.ui.main_window))
 
-    def test_windows_have_correct_icon(self):
+    def test_main_window_have_correct_icon_list(self):
         """Every window has the icon set."""
-        for w in self.ui.windows:
-            self.assertEqual(w.get_icon(), self.ui._icon)
+        self.assertEqual(len(self.ui.main_window.get_icon_list()),
+                         len(self.ui._icons.values()))
+
+    def test_every_window_has_correct_list(self):
+        """The default icon list is set."""
+        self.patch(gtk, 'window_set_default_icon_list', self._set_called)
+        MagicicadaUI(syncdaemon_class=FakedSyncdaemon)
+        self.assertEqual(len(self._called[0]), len(self.ui._icons.values()))
 
     def test_start_connect_are_visible(self):
         """Start and Connect buttons are visible."""
@@ -456,6 +512,8 @@ class _MagicicadaUIQueueTestCase(MagicicadaUITestCase):
     """Abstratc UI test cases for queue tree views."""
 
     name = None
+    data_type = QueueData
+    mapping = enumerate(['operation', 'path', 'share', 'node'])
 
     def setUp(self):
         """Init."""
@@ -468,15 +526,8 @@ class _MagicicadaUIQueueTestCase(MagicicadaUITestCase):
                                   'on_%s_queue_changed' % self.name)
         self.set_sd_queue = lambda q: \
                             setattr(self.ui.sd, '%s_queue' % self.name, q)
-        self.queue_store = getattr(self.ui, '%sq_store' % self.name)
-        self.queue_view = getattr(self.ui, '%sq_view' % self.name)
-
-    def build_some_data(self, limit=5):
-        """Build some data to act as queue data."""
-        kwargs = dict(data_type=QueueData, limit=limit)
-        res = super(_MagicicadaUIQueueTestCase, self).build_some_data(**kwargs)
-        # operation path share node
-        return res
+        self.store = getattr(self.ui, '%sq_store' % self.name)
+        self.view = getattr(self.ui, '%sq_view' % self.name)
 
     def expected_markup(self, value):
         """Return the markup for row at index 'i'."""
@@ -491,10 +542,9 @@ class _MagicicadaUIQueueTestCase(MagicicadaUITestCase):
                  if must_highlight and value is not None else value
         return result
 
-    def assert_store_correct(self, store, items):
+    def assert_store_correct(self, items):
         """Test that 'store' has 'items' as content."""
-        mapping = enumerate(['operation', 'path', 'share', 'node'])
-        args = (store, items, mapping, self.expected_markup)
+        args = (items, self.expected_markup)
         super(_MagicicadaUIQueueTestCase, self).assert_store_correct(*args)
 
     def assert_current_processing_row_is_different(self):
@@ -508,8 +558,8 @@ class _MagicicadaUIQueueTestCase(MagicicadaUITestCase):
         markup = self.expected_markup
         expected = tuple(markup(getattr(item, attr)) for attr in attrs)
 
-        iter_root = self.queue_store.get_iter_root()
-        actual = self.queue_store.get(iter_root, *xrange(len(attrs)))
+        iter_root = self.store.get_iter_root()
+        actual = self.store.get(iter_root, *xrange(len(attrs)))
 
         msg = 'first row for %s queue must be %s (got %s instead)' % \
               (self.name, expected, actual)
@@ -524,30 +574,30 @@ class _MagicicadaUIQueueTestCase(MagicicadaUITestCase):
     @skip_abstract_class
     def test_model_is_binded(self):
         """List store is binded."""
-        actual = self.queue_view.get_model()
+        actual = self.view.get_model()
         msg = 'model for view %s differs from %s'
-        self.assertEqual(self.queue_store, actual,
-                         msg % (self.name, self.queue_store))
+        self.assertEqual(self.store, actual,
+                         msg % (self.name, self.store))
 
     @skip_abstract_class
     def test_on_queue_changed_updates_view(self):
         """On queue changed the view is updated."""
         items = self.build_some_data()
         self.sd_changed(items)
-        self.assert_store_correct(self.queue_store, items)
+        self.assert_store_correct(items)
 
     @skip_abstract_class
     def test_on_queue_changed_handles_none(self):
         """On queue changed handles None as items."""
         self.sd_changed(None)
-        self.assert_store_correct(self.queue_store, [])
+        self.assert_store_correct([])
 
     @skip_abstract_class
     def test_on_queue_changed_handles_an_item_none(self):
         """On queue changed handles None as items."""
         items = [QueueData(operation='Test', path='', share=None, node=None)]
         self.sd_changed(items)
-        self.assert_store_correct(self.queue_store, items)
+        self.assert_store_correct(items)
 
     @skip_abstract_class
     def test_model_is_cleared_before_updating(self):
@@ -557,17 +607,17 @@ class _MagicicadaUIQueueTestCase(MagicicadaUITestCase):
 
         items = self.build_some_data()
         self.sd_changed(items)
-        self.assertEqual(len(self.queue_store), len(items))
+        self.assertEqual(len(self.store), len(items))
 
     @skip_abstract_class
     def test_view_is_enabled_if_disabled_on_changed(self):
         """The tree view is enabled on changed if it was disabled."""
-        self.assertFalse(self.queue_view.is_sensitive(),
+        self.assertFalse(self.view.is_sensitive(),
                          'Tree view must be disabled by default.')
         items = self.build_some_data()
         self.sd_changed(items)
 
-        self.assertTrue(self.queue_view.is_sensitive(),
+        self.assertTrue(self.view.is_sensitive(),
                         'Tree view must be enabled on changed.')
 
     @skip_abstract_class
@@ -578,7 +628,7 @@ class _MagicicadaUIQueueTestCase(MagicicadaUITestCase):
 
         self.ui.update()
 
-        self.assert_store_correct(self.queue_store, data)
+        self.assert_store_correct(data)
 
     @skip_abstract_class
     def test_on_stopped_updates_queue(self):
@@ -594,7 +644,7 @@ class _MagicicadaUIQueueTestCase(MagicicadaUITestCase):
         label = '%sq_label' % self.name
         actual = getattr(self.ui, label).get_text()
         expected = '%s Queue (%i)' % (self.name.capitalize(),
-                                      len(self.queue_store))
+                                      len(self.store))
         msg = '%s should be %s (got %s instead)'
         self.assertEqual(expected, actual, msg % (label, expected, actual))
 
@@ -608,13 +658,13 @@ class _MagicicadaUIQueueTestCase(MagicicadaUITestCase):
         items = self.build_some_data()
         self.set_sd_queue(items)
         self.do_start()
-        self.assert_store_correct(self.queue_store, items)
+        self.assert_store_correct(items)
 
         items = items[:len(items) / 2]
         self.set_sd_queue(items)
         self.ui.on_stop_clicked(self.ui.stop)
         self.ui.on_stopped()
-        self.assert_store_correct(self.queue_store, items)
+        self.assert_store_correct(items)
 
     @skip_abstract_class
     def test_current_processing_row_is_different_if_online(self):
@@ -945,74 +995,40 @@ class _MagicicadaUIVolumeTestCase(MagicicadaUITestCase):
         if self.name is None:
             return
         self.volume = getattr(self.ui, self.name)
-        self.volume_store = getattr(self.ui, '%s_store' % self.name)
-        self.volume_view = getattr(self.ui, '%s_view' % self.name)
+        self.store = getattr(self.ui, '%s_store' % self.name)
+        self.view = getattr(self.ui, '%s_view' % self.name)
         self.volume_dialog_name = '%s_dialog' % self.name
         self.volume_dialog = getattr(self.ui, self.volume_dialog_name)
         self.on_volume_clicked = getattr(self.ui, 'on_%s_clicked' % self.name)
         self.volume_close = getattr(self.ui, '%s_close' % self.name)
 
-    def build_some_data(self, limit=5):
-        """Build some data to act as volume."""
-        kwargs = dict(data_type=self.data_type, limit=limit)
-        r = super(_MagicicadaUIVolumeTestCase, self).build_some_data(**kwargs)
-        return r
-
-    def assert_store_correct(self, store, items):
-        """Test that 'store' has 'items' as content."""
-        args = (store, items, self.mapping)
-        super(_MagicicadaUIVolumeTestCase, self).assert_store_correct(*args)
-
     def assert_widget_availability(self, enabled=True):
         """Check volume availability according to 'enabled'."""
         s = super(_MagicicadaUIVolumeTestCase, self)
         s.assert_widget_availability(self.name, enabled)
 
-    def assert_sort_order_correct(self, column, i, expected_order):
-        """Check that sort order is correctly set."""
-        msg0 = 'Store sort id must be %r (got %r instead).'
-        msg1 = 'Store sort order must be %r (got %r instead).'
-        msg3 = 'Column sort order must be %r (got %r instead).'
-
-        actual_id, actual_order = self.volume_store.get_sort_column_id()
-
-        # store sort column id and order
-        self.assertEqual(i, actual_id, msg0 % (i, actual_id))
-        self.assertEqual(expected_order, actual_order,
-                         msg1 % (expected_order, actual_order))
-
-        # column sort order
-        actual_order = column.get_sort_order()
-        self.assertEqual(expected_order, actual_order,
-                         msg3 % (expected_order, actual_order))
-
-    def assert_sort_indicator_correct(self, column):
-        """Check that sort indicator is correctly set."""
-        msg = 'Column %s must have sort indicator %s.'
-        colname = column.get_name()
-        # column sort indicator
-        self.assertTrue(column.get_sort_indicator(), msg % (colname, 'on'))
-
-        # all the other columns must not have the sort indicator on
-        for other_column in self.volume_view.get_columns():
-            if other_column.get_name() == colname:
-                continue
-            self.assertFalse(other_column.get_sort_indicator(),
-                             msg % (other_column.get_name(), 'off'))
+    @skip_abstract_class
+    def test_initial_data_ready_callback_connected(self):
+        """The callback 'on_initial_data_ready' is connected to SD."""
+        self.assertEqual(self.ui.sd.on_initial_data_ready_callback,
+                         self.ui.on_initial_data_ready,
+                         "on_initial_data_ready should be connected.")
 
     @skip_abstract_class
-    def test_volume_are_disabled_until_started(self):
-        """Folders and shares are disabled until online."""
+    def test_volume_are_disabled_until_initial_data_ready(self):
+        """Folders and shares are disabled until data ready."""
         # disabled at startup
         self.assert_widget_availability(enabled=False)
 
-        # enabled when started
-        self.do_start()
+        # enabled when initial data ready
+        self.ui.on_initial_data_ready()
         self.assert_widget_availability(enabled=True)
 
     @skip_abstract_class
     def test_volume_are_enabled_until_stopped(self):
         """Folders and shares are enabled until offline."""
+        self.ui.on_initial_data_ready()
+
         self.do_connect()
         self.assert_widget_availability(enabled=True)
 
@@ -1057,7 +1073,7 @@ class _MagicicadaUIVolumeTestCase(MagicicadaUITestCase):
             """Perform the test per se before closing the dialog."""
             self.assertTrue(self.ui.widget_is_visible(self.volume_dialog),
                             '%s should be visible.' % self.volume_dialog_name)
-            self.assert_store_correct(self.volume_store, items)
+            self.assert_store_correct(items)
 
         items = self.build_some_data()
         setattr(self.ui.sd, self.name, items)
@@ -1077,7 +1093,7 @@ class _MagicicadaUIVolumeTestCase(MagicicadaUITestCase):
             """Perform the test per se before closing the dialog."""
             self.assertTrue(self.ui.widget_is_visible(self.volume_dialog),
                             '%s should be visible.' % self.volume_dialog_name)
-            self.assert_store_correct(self.volume_store, items)
+            self.assert_store_correct(items)
 
         items = self.build_some_data()
         setattr(self.ui.sd, self.name, items)
@@ -1094,7 +1110,7 @@ class _MagicicadaUIVolumeTestCase(MagicicadaUITestCase):
     def test_on_volume_clicked_handles_none(self):
         """On volume clicked handles None as items."""
         setattr(self.ui.sd, self.name, None)
-        test = lambda: self.assert_store_correct(self.volume_store, [])
+        test = lambda: self.assert_store_correct([])
         gobject.timeout_add(100, close_dialog,
                             (self.volume_dialog, test))
         self.on_volume_clicked(self.volume)
@@ -1110,36 +1126,36 @@ class _MagicicadaUIVolumeTestCase(MagicicadaUITestCase):
     def test_volume_columns_not_sorted_at_start(self):
         """Test volume columns are not sorted at start."""
         msg = 'Column %s must not have the sort indicator on.'
-        for col in self.volume_view.get_columns():
+        for col in self.view.get_columns():
             self.assertFalse(col.get_sort_indicator(), msg % col.get_name())
 
     @skip_abstract_class
     def test_volume_columns_are_clickable(self):
         """Test volume columns are clickable."""
         msg = 'Column %s must be clickable.'
-        for col in self.volume_view.get_columns():
+        for col in self.view.get_columns():
             self.assertTrue(col.get_clickable(), msg % col.get_name())
 
     @skip_abstract_class
     def test_volume_columns_clicked_signal(self):
         """Test volume columns clicks signal is properly connected."""
         msg = 'Column %s must be connected to on_store_sort_column_changed.'
-        for col in self.volume_view.get_columns():
+        for col in self.view.get_columns():
             self.assertTrue(col.get_clickable(), msg % col.get_name())
 
     @skip_abstract_class
     def test_volume_sorting(self):
         """Test volume panel can be re-sorted."""
-        for i, col in enumerate(self.volume_view.get_columns()):
+        for idx, col in enumerate(self.view.get_columns()):
             col.clicked()  # click on the column
-            self.assert_sort_order_correct(col, i, gtk.SORT_ASCENDING)
+            self.assert_sort_order_correct(col, idx, gtk.SORT_ASCENDING)
             self.assert_sort_indicator_correct(col)
 
             col.clicked()  # click on the column, sort order must change
-            self.assert_sort_order_correct(col, i, gtk.SORT_DESCENDING)
+            self.assert_sort_order_correct(col, idx, gtk.SORT_DESCENDING)
 
             col.clicked()  # click again, sort order must be the first one
-            self.assert_sort_order_correct(col, i, gtk.SORT_ASCENDING)
+            self.assert_sort_order_correct(col, idx, gtk.SORT_ASCENDING)
 
 
 class MagicicadaUIFoldersTestCase(_MagicicadaUIVolumeTestCase):
@@ -1173,7 +1189,7 @@ class _MagicicadaUISharesTestCase(_MagicicadaUIVolumeTestCase):
                 i = int(kwargs['free_bytes'])
                 kwargs['free_bytes'] = humanize_bytes(i, precision=2)
             item = self.data_type(**kwargs)
-            self.assert_store_correct(self.volume_store, [item])
+            self.assert_store_correct([item])
 
         gobject.timeout_add(100, close_dialog,
                             (self.volume_dialog, test))
@@ -1266,17 +1282,19 @@ class MagicicadaUIMetadataTestCase(MagicicadaUITestCase):
         msg = 'buffer content must be %s (got %s instead).'
         self.assertEqual(actual, expected, msg % (expected, actual))
 
-    def test_metadata_are_disabled_until_started(self):
-        """Metadata button is disabled until online."""
+    def test_metadata_are_disabled_until_initial_data_ready(self):
+        """Metadata button is disabled until initial data."""
         # disabled at startup
         self.assert_widget_availability(enabled=False)
 
-        # enabled when started
-        self.do_start()
+        # enabled when initial data ready
+        self.ui.on_initial_data_ready()
         self.assert_widget_availability(enabled=True)
 
     def test_metadata_are_enabled_until_stopped(self):
         """Metadata button is enabled until offline."""
+        self.ui.on_initial_data_ready()
+
         self.do_connect()
         self.assert_widget_availability(enabled=True)
 
@@ -1515,11 +1533,12 @@ class MagicicadaLoggingTestCase(MagicicadaUITestCase):
         self.memento.setLevel(logging.DEBUG)
         logger = logging.getLogger('magicicada.ui')
         logger.addHandler(self.memento)
+        logger.setLevel(logging.DEBUG)
 
-    def assert_function_logs(self, func, *args, **kwargs):
-        """Check 'funcion' logs its inputs as DEBUG."""
+    def assert_function_logs(self, level, func, *args, **kwargs):
+        """Check 'funcion' logs its inputs as 'level'."""
         name = func.__name__
-        msg = '%s must be logged as DEBUG'
+        msg = '%s must be logged with level %r'
         try:
             func(*args, **kwargs)
         except Exception:  # pylint: disable=E0501, W0703
@@ -1528,33 +1547,201 @@ class MagicicadaLoggingTestCase(MagicicadaUITestCase):
                             'function (%s) must be logged as ERROR' % name)
             self.assertTrue(self.memento.check_error(exc),
                             'sys.exc_info (%s) must be logged as ERROR' % exc)
-        self.assertTrue(self.memento.check_debug(name), msg % name)
+
+        memento_func = getattr(self.memento, 'check_%s' % level.lower())
+        self.assertTrue(memento_func(name), msg % (name, level))
         for arg in args:
-            self.assertTrue(self.memento.check_debug(str(arg)), msg % arg)
+            self.assertTrue(memento_func(str(arg)), msg % (arg, level))
         for key, val in kwargs.iteritems():
             arg = "'%s': %r" % (key, val)
-            self.assertTrue(self.memento.check_debug(arg), msg % arg)
+            self.assertTrue(memento_func(arg), msg % (arg, level))
 
     def test_on_shares_clicked_logs(self):
         """Check _on_shares_clicked logs properly."""
         args = ([0, object(), 'test', {}], object())
         kwargs = dict(dialog=object())
-        self.assert_function_logs(self.ui._on_shares_clicked, *args, **kwargs)
+        self.assert_function_logs(logging.getLevelName(logging.DEBUG),
+                                  self.ui._on_shares_clicked, *args, **kwargs)
 
     def test_on_status_changed_logs(self):
         """Check _on_status_changed logs properly."""
         args = ('test status', 'status description', True, False, True)
         kwargs = dict(queues='bla', connection=None)
-        self.assert_function_logs(self.ui.on_status_changed, *args, **kwargs)
+        self.assert_function_logs(logging.getLevelName(logging.DEBUG),
+                                  self.ui.on_status_changed, *args, **kwargs)
 
     def test_on_queue_changed_logs(self):
         """Check _on_queue_changed logs properly."""
         args = ('meta',)
         kwargs = dict(items=[0, object(), 'test', {}], must_highlight=True)
-        self.assert_function_logs(self.ui._on_queue_changed, *args, **kwargs)
+        self.assert_function_logs(logging.getLevelName(logging.DEBUG),
+                                  self.ui._on_queue_changed, *args, **kwargs)
 
     def test_on_metadata_ready_logs(self):
         """Check on_metadata_ready logs properly."""
         args = ()
         kwargs = dict(path='test', metadata=True)
-        self.assert_function_logs(self.ui.on_metadata_ready, *args, **kwargs)
+        self.assert_function_logs(logging.getLevelName(logging.DEBUG),
+                                  self.ui.on_metadata_ready, *args, **kwargs)
+
+    def test_on_initial_data_ready_logs(self):
+        """Check on_initial_data_ready logs properly."""
+        args = ()
+        kwargs = dict(path='test', metadata=True)
+        self.assert_function_logs(logging.getLevelName(logging.INFO),
+                                  self.ui.on_initial_data_ready,
+                                  *args, **kwargs)
+
+
+class PublicFilesTestCase(MagicicadaUITestCase):
+    """UI test cases for public files."""
+
+    name = 'public_files'
+    data_type = PublicFilesData
+    mapping = enumerate(['path', 'public_url', 'volume', 'node'])
+
+    def setUp(self):
+        """Init."""
+        super(PublicFilesTestCase, self).setUp()
+        self.store = self.ui.public_files_store
+        self.view = self.ui.public_files_view
+
+    def assert_widget_availability(self, enabled=True, msg=None):
+        """Check widget availability according to 'enabled'."""
+        widget = self.ui.public_files
+        self.assertTrue(self.ui.widget_is_visible(widget), 'must be visible')
+
+        sensitive = widget.is_sensitive()
+        if msg is None:
+            msg = 'must %sbe sensitive' % ('' if enabled else 'not ',)
+        self.assertTrue(sensitive if enabled else not sensitive, msg)
+
+    def test_initial_data_ready_callback_connected(self):
+        """The callback 'on_initial_online_data_ready' is connected to SD."""
+        self.assertEqual(self.ui.sd.on_initial_online_data_ready_callback,
+                         self.ui.on_initial_online_data_ready,
+                         "on_initial_online_data_ready should be connected.")
+
+    def test_disabled_until_initial_online_data_ready(self):
+        """Widget is disabled until initial online data ready."""
+        self.assert_widget_availability(enabled=False,
+                                        msg='must be disabled at startup')
+
+        self.ui.on_initial_online_data_ready()
+        msg = 'must be enabled when initial data ready'
+        self.assert_widget_availability(enabled=True, msg=msg)
+
+    def test_enabled_until_stopped(self):
+        """Widget is enabled until stopped."""
+        self.ui.on_initial_online_data_ready()
+
+        self.do_connect()
+        self.assert_widget_availability(enabled=True)
+
+        self.ui.on_online()
+        self.assert_widget_availability(enabled=True)
+
+        self.ui.on_offline()
+        self.assert_widget_availability(enabled=True)
+
+        self.ui.on_stopped()
+        self.assert_widget_availability(enabled=False)
+
+    def test_public_files_close_emits_response_close(self):
+        """Test close button emits RESPONSE_CLOSE when clicked."""
+        self.patch(self.ui.public_files_dialog, 'response', self._set_called)
+        self.ui.public_files_close.clicked()
+        self.assertEqual(self._called, ((gtk.RESPONSE_CLOSE,), {}),
+                         'close button should emit RESPONSE_CLOSE.')
+
+    def test_on_public_files_clicked_runs_the_dialog(self):
+        """Test on_public_files_clicked run the public_files_dialog."""
+        self.patch(self.ui.public_files_dialog, 'run', self._set_called)
+        self.ui.sd.public_files = {}
+
+        visible = self.ui.widget_is_visible(self.ui.public_files_dialog)
+        self.assertFalse(visible, 'dialog should not be visible.')
+
+        self.ui.on_public_files_clicked(self.ui.public_files)
+
+        self.assertEqual(self._called, ((), {}))
+
+    def test_on_public_files_clicked_hides_the_dialog(self):
+        """Test on_public_files_clicked hides the public_files_dialog."""
+        self.patch(self.ui.public_files_dialog, 'run', lambda: None)  # no run
+        self.patch(self.ui.public_files_dialog, 'hide', self._set_called)
+        self.ui.sd.public_files = {}
+
+        self.ui.on_public_files_clicked(self.ui.public_files)
+
+        self.assertEqual(self._called, ((), {}))
+
+    def test_on_public_files_clicked_grabs_info_from_backend(self):
+        """Test on_public_files_clicked."""
+        self.patch(self.ui.public_files_dialog, 'run', lambda: None)  # no run
+        self.ui.sd.public_files = SAMPLE_PUBLIC_FILES
+
+        self.ui.on_public_files_clicked(self.ui.public_files)
+
+        self.assert_store_correct(SAMPLE_PUBLIC_FILES.values())
+
+    def test_on_public_files_clicked_twice(self):
+        """Test on_public_files_clicked twice."""
+        self.patch(self.ui.public_files_dialog, 'run', lambda: None)  # no run
+        self.ui.sd.public_files = SAMPLE_PUBLIC_FILES
+
+        self.ui.on_public_files_clicked(self.ui.public_files)
+        self.ui.on_public_files_clicked(self.ui.public_files)
+
+        self.assert_store_correct(SAMPLE_PUBLIC_FILES.values())
+
+    def test_on_public_files_clicked_handles_none(self):
+        """On public_files clicked handles None as items."""
+        self.patch(self.ui.public_files_dialog, 'run', lambda: None)  # no run
+        self.ui.sd.public_files = None
+
+        self.ui.on_public_files_clicked(self.ui.public_files)
+
+        self.assert_store_correct([])
+
+    def test_dialog_properties(self):
+        """The dialog has correct properties."""
+        title = self.name.replace('_', ' ').capitalize()
+        self.assert_dialog_properties(dialog_name='public_files_dialog',
+                                      title=title)
+
+    def test_columns_not_sorted_at_start(self):
+        """Test columns are not sorted at start."""
+        msg = 'Column %s must not have the sort indicator on.'
+        for col in self.ui.public_files_view.get_columns():
+            self.assertFalse(col.get_sort_indicator(), msg % col.get_name())
+
+    def test_columns_are_clickable(self):
+        """Test columns are clickable."""
+        msg = 'Column %s must be clickable.'
+        for col in self.ui.public_files_view.get_columns():
+            self.assertTrue(col.get_clickable(), msg % col.get_name())
+
+    def test_columns_clicked_signal(self):
+        """Test columns clicked signal is properly connected."""
+        msg = 'Column %s must be connected to on_store_sort_column_changed.'
+        for col in self.ui.public_files_view.get_columns():
+            self.assertTrue(col.get_clickable(), msg % col.get_name())
+
+    def test_sorting(self):
+        """Test public files panel can be re-sorted."""
+        self.patch(self.ui.public_files_dialog, 'run', lambda: None)  # no run
+        self.ui.sd.public_files = SAMPLE_PUBLIC_FILES
+
+        self.ui.on_public_files_clicked(self.ui.public_files)
+
+        for idx, col in enumerate(self.ui.public_files_view.get_columns()):
+            col.clicked()  # click on the column
+            self.assert_sort_order_correct(col, idx, gtk.SORT_ASCENDING)
+            self.assert_sort_indicator_correct(col)
+
+            col.clicked()  # click on the column, sort order must change
+            self.assert_sort_order_correct(col, idx, gtk.SORT_DESCENDING)
+
+            col.clicked()  # click again, sort order must be the first one
+            self.assert_sort_order_correct(col, idx, gtk.SORT_ASCENDING)
