@@ -22,7 +22,13 @@ import logging
 import os
 import unittest
 
-from magicicada.syncdaemon import SyncDaemon, State
+from magicicada.dbusiface import PublicFilesData, ShareOperationError
+from magicicada.syncdaemon import (
+    ASKING_IDLE,
+    mandatory_callback,
+    State,
+    SyncDaemon,
+)
 from magicicada.tests.helpers import MementoHandler
 
 from twisted.trial.unittest import TestCase as TwistedTestCase
@@ -32,18 +38,23 @@ from twisted.internet import defer, reactor
 # It's ok to access private data in the test suite
 # pylint: disable=W0212
 
+# Lambda may not be necessary
+# pylint: disable=W0108
+
 
 class FakeDBusInterface(object):
     """Fake DBus Interface, for SD to not use dbus at all during tests."""
 
     fake_sd_started = False
+    fake_pf_data = PublicFilesData(volume='v', node='n',
+                                   path='p', public_url='u')
+    fake_share_response = None
 
     def __init__(self, sd):
         pass
 
     def shutdown(self):
         """Fake shutdown."""
-        pass
 
     def get_status(self):
         """Fake status."""
@@ -58,10 +69,18 @@ class FakeDBusInterface(object):
     start = quit = connect = disconnect = get_folders
     get_shares_to_me = get_shares_to_others = get_folders
 
+    def get_public_files(self):
+        """Fake public files."""
+        return defer.succeed([self.fake_pf_data])
+
     def is_sd_started(self):
         """Fake response."""
         return self.fake_sd_started
 
+    def accept_share(self, share_id):
+        """Fake accept share."""
+        return self.fake_share_response
+
 
 class BaseTest(TwistedTestCase):
     """Base test with a SD."""
@@ -70,6 +89,11 @@ class BaseTest(TwistedTestCase):
 
     def setUp(self):
         """Set up."""
+        self.hdlr = MementoHandler()
+        self.hdlr.setLevel(logging.DEBUG)
+        logger = logging.getLogger('magicicada.syncdaemon')
+        logger.addHandler(self.hdlr)
+        logger.setLevel(logging.DEBUG)
         self.sd = SyncDaemon(FakeDBusInterface)
 
     def tearDown(self):
@@ -77,9 +101,44 @@ class BaseTest(TwistedTestCase):
         self.sd.shutdown()
 
 
+class MandatoryCallbackTests(BaseTest):
+    """Tests for the mandatory callback generic function."""
+
+    def test_log_function_name(self):
+        """Log the function name."""
+        some_function = mandatory_callback('bar')
+        some_function()
+        self.assertTrue(self.hdlr.check_warning(
+                        "Callback called but was not assigned", "bar"))
+
+    def test_log_args(self):
+        """Log the arguments."""
+        some_function = mandatory_callback('bar')
+        some_function(1, 2, b=45)
+        self.hdlr.debug = True
+        self.assertTrue(self.hdlr.check_warning(
+                        "Callback called but was not assigned",
+                        "1", "2", "'b': 45"))
+
+
 class InitialDataTests(unittest.TestCase):
     """Tests for initial data gathering."""
 
+    def setUp(self):
+        """Set up the test."""
+        self.sd = SyncDaemon(FakeDBusInterface)
+
+        self.offline_called = False
+        self.sd.on_initial_data_ready_callback = lambda: setattr(self,
+                                                        'offline_called', True)
+        self.online_called = False
+        self.sd.on_initial_online_data_ready_callback = lambda: setattr(self,
+                                                        'online_called', True)
+
+    def tearDown(self):
+        """Tear down the test."""
+        self.sd.shutdown()
+
     def test_called_by_start(self):
         """Check that start calls get initial data."""
         sd = SyncDaemon(FakeDBusInterface)
@@ -135,6 +194,68 @@ class InitialDataTests(unittest.TestCase):
         sd._get_initial_data()
         self.assertEqual(len(called), 3)
 
+    def test_public_files_info(self):
+        """Check we get the public files info at start."""
+        sd = SyncDaemon(FakeDBusInterface)
+        fake_data = FakeDBusInterface.fake_pf_data
+        sd._get_initial_data()
+        self.assertEqual(sd.public_files[fake_data.node], fake_data)
+
+    def test_all_ready(self):
+        """All data is ready."""
+        self.sd._get_initial_data()
+        self.assertTrue(self.offline_called)
+        self.assertTrue(self.online_called)
+
+    def test_no_public_files(self):
+        """Initial gathering is stuck in public files."""
+        self.sd.dbus.get_public_files = lambda: defer.Deferred()
+        self.sd._get_initial_data()
+        self.assertTrue(self.offline_called)
+        self.assertFalse(self.online_called)
+
+    def test_no_shares_to_others(self):
+        """Initial gathering is stuck in shares to others."""
+        self.sd.dbus.get_shares_to_others = lambda: defer.Deferred()
+        self.sd._get_initial_data()
+        self.assertFalse(self.offline_called)
+        self.assertFalse(self.online_called)
+
+    def test_no_shares_to_me(self):
+        """Initial gathering is stuck in shares to me."""
+        self.sd.dbus.get_shares_to_me = lambda: defer.Deferred()
+        self.sd._get_initial_data()
+        self.assertFalse(self.offline_called)
+        self.assertFalse(self.online_called)
+
+    def test_no_folders(self):
+        """Initial gathering is stuck in folders."""
+        self.sd.dbus.get_folders = lambda: defer.Deferred()
+        self.sd._get_initial_data()
+        self.assertFalse(self.offline_called)
+        self.assertFalse(self.online_called)
+
+    def test_no_meta_queue(self):
+        """Initial gathering is stuck in meta queue."""
+        self.sd.dbus.get_meta_queue = lambda: defer.Deferred()
+        self.sd._get_initial_data()
+        self.assertFalse(self.offline_called)
+        self.assertFalse(self.online_called)
+
+    def test_no_content_queue(self):
+        """Initial gathering is stuck in content queue."""
+        self.sd.dbus.get_content_queue = lambda: defer.Deferred()
+        self.sd._get_initial_data()
+        self.assertFalse(self.offline_called)
+        self.assertFalse(self.online_called)
+
+    def test_no_status(self):
+        """Initial gathering is stuck in status."""
+        self.sd.dbus.get_status = lambda: defer.Deferred()
+        self.sd._get_initial_data()
+        self.assertFalse(self.offline_called)
+        self.assertFalse(self.online_called)
+
 
 class StatusChangedTests(BaseTest):
     """Simple signals checking."""
@@ -175,7 +296,7 @@ class StatusChangedTests(BaseTest):
                                      False, 'queues', 'connection')
         return deferred
 
-    def test_status_changed_affects_cuurent_status(self):
+    def test_status_changed_affects_current_status(self):
         """Make changes to see how status are reflected."""
         # one set of values
         self.sd.on_sd_status_changed('name1', 'description1', False, True,
@@ -322,24 +443,115 @@ class ContentQueueChangedTests(BaseTest):
         self.sd.on_sd_content_queue_changed()
         self.assertEqual(called, [['foo']])
 
-    def test_CQ_state_nothing(self):
+    def test_cq_state_nothing(self):
         """Check the ContentQueue info, being nothing."""
         self.sd.dbus.get_content_queue = lambda: defer.succeed([])
         self.sd.on_sd_content_queue_changed()
         self.assertEqual(self.sd.content_queue, [])
 
-    def test_CQ_state_one(self):
+    def test_cq_state_one(self):
         """Check the ContentQueue info, being one."""
         self.sd.dbus.get_content_queue = lambda: defer.succeed(['foo'])
         self.sd.on_sd_content_queue_changed()
         self.assertEqual(self.sd.content_queue, ['foo'])
 
-    def test_CQ_state_two(self):
+    def test_cq_state_two(self):
         """Check the ContentQueue info, two."""
         self.sd.dbus.get_content_queue = lambda: defer.succeed(['foo', 'bar'])
         self.sd.on_sd_content_queue_changed()
         self.assertEqual(self.sd.content_queue, ['foo', 'bar'])
 
+    def test_cq_multiple_without_response_having_two(self):
+        """Behave correctly on two-changes burst."""
+        d1 = defer.Deferred()
+        d2 = defer.Deferred()
+        called = []
+
+        def fake():
+            """Fake."""
+            d = d2 if called else d1
+            called.append(None)
+            return d
+        self.sd.dbus.get_content_queue = fake
+
+        # call twice
+        self.sd.on_sd_content_queue_changed()
+        self.sd.on_sd_content_queue_changed()
+
+        # dbus function should be called once
+        self.assertEqual(len(called), 1)
+
+        # first call returns
+        d1.callback(['foo'])
+        d2.callback(['bar'])
+
+        # it should be called once more only
+        self.assertEqual(len(called), 2)
+
+    def test_cq_multiple_without_response_having_three(self):
+        """Behave correctly on three-changes burst."""
+        d1 = defer.Deferred()
+        d2 = defer.Deferred()
+        called = []
+
+        def fake():
+            """Fake."""
+            d = d2 if called else d1
+            called.append(None)
+            return d
+        self.sd.dbus.get_content_queue = fake
+
+        # call thrice
+        self.sd.on_sd_content_queue_changed()
+        self.sd.on_sd_content_queue_changed()
+        self.sd.on_sd_content_queue_changed()
+
+        # dbus function should be called once
+        self.assertEqual(len(called), 1)
+
+        # first call returns
+        d1.callback(['foo'])
+        d2.callback(['bar'])
+
+        # it should be called once more only
+        self.assertEqual(len(called), 2)
+
+    def test_cq_multiple_without_response_having_more_later(self):
+        """Behave correctly on changes burst delayed."""
+        d1 = defer.Deferred()
+        d2 = defer.Deferred()
+        d3 = defer.Deferred()
+        defs = [d1, d2, d3]
+        called = []
+
+        def fake():
+            """Fake."""
+            d = defs[len(called)]
+            called.append(None)
+            return d
+        self.sd.dbus.get_content_queue = fake
+
+        # call some times, dbus function should be called once
+        self.sd.on_sd_content_queue_changed()
+        self.sd.on_sd_content_queue_changed()
+        self.sd.on_sd_content_queue_changed()
+        self.assertEqual(len(called), 1)
+
+        # first call returns
+        d1.callback(['foo'])
+
+        # while calling second time, generate more requests
+        self.sd.on_sd_content_queue_changed()
+        self.sd.on_sd_content_queue_changed()
+        self.sd.on_sd_content_queue_changed()
+
+        # second call returns
+        d2.callback(['foo'])
+        d3.callback(['foo'])
+
+        # it should be called three times total
+        self.assertEqual(len(called), 3)
+
 
 class MetaQueueChangedTests(BaseTest):
     """Check the MetaQueueChanged handling."""
@@ -526,6 +738,8 @@ class MetaQueueChangedTests(BaseTest):
 
     def test_mq_caller_is_reset_last_time(self):
         """When MQ is polled last time, the caller should be back to None."""
+        # Module 'twisted.internet.reactor' has no 'callLater' member
+        # pylint: disable=E1101
         self.sd._mqcaller = reactor.callLater(100, lambda: None)
         self.sd.current_state.set(name='QUEUE_MANAGER',
                                   queues='WORKING_ON_CONTENT')
@@ -550,6 +764,8 @@ class MetaQueueChangedTests(BaseTest):
                                              'WORKING_ON_CONTENT', 'connect')
 
                 # allow time to see if a mistaken call happens
+                # Module 'twisted.internet.reactor' has no 'callLater' member
+                # pylint: disable=E1101
                 reactor.callLater(.5, deferred.callback, True)
             elif len(calls) == 4:
                 pass    # last call after state changed
@@ -566,24 +782,132 @@ class MetaQueueChangedTests(BaseTest):
         deferred = defer.Deferred()
         return deferred
 
-    def test_MQ_state_nothing(self):
+    def test_mq_state_nothing(self):
         """Check the MetaQueue info, being nothing."""
         self.sd.dbus.get_meta_queue = lambda: defer.succeed([])
         self.sd._check_mq()
         self.assertEqual(self.sd.meta_queue, [])
 
-    def test_MQ_state_one(self):
+    def test_mq_state_one(self):
         """Check the MetaQueue info, being one."""
         self.sd.dbus.get_meta_queue = lambda: defer.succeed(['foo'])
         self.sd._check_mq()
         self.assertEqual(self.sd.meta_queue, ['foo'])
 
-    def test_MQ_state_two(self):
+    def test_mq_state_two(self):
         """Check the MetaQueue info, two."""
         self.sd.dbus.get_meta_queue = lambda: defer.succeed(['foo', 'bar'])
         self.sd._check_mq()
         self.assertEqual(self.sd.meta_queue, ['foo', 'bar'])
 
+    def test_mq_multiple_without_response_having_two(self):
+        """Behave correctly on two-changes burst."""
+        d1 = defer.Deferred()
+        d2 = defer.Deferred()
+        called = []
+
+        def fake():
+            """Fake."""
+            d = d2 if called else d1
+            called.append(None)
+            return d
+        self.sd.dbus.get_meta_queue = fake
+
+        # call twice
+        self.sd.on_sd_meta_queue_changed()
+        self.sd.on_sd_meta_queue_changed()
+
+        # dbus function should be called once
+        self.assertEqual(len(called), 1)
+
+        # first call returns
+        d1.callback(['foo'])
+        d2.callback(['bar'])
+
+        # it should be called once more only
+        self.assertEqual(len(called), 2)
+
+    def test_mq_multiple_without_response_having_three(self):
+        """Behave correctly on three-changes burst."""
+        d1 = defer.Deferred()
+        d2 = defer.Deferred()
+        called = []
+
+        def fake():
+            """Fake."""
+            d = d2 if called else d1
+            called.append(None)
+            return d
+        self.sd.dbus.get_meta_queue = fake
+
+        # call thrice
+        self.sd.on_sd_meta_queue_changed()
+        self.sd.on_sd_meta_queue_changed()
+        self.sd.on_sd_meta_queue_changed()
+
+        # dbus function should be called once
+        self.assertEqual(len(called), 1)
+
+        # first call returns
+        d1.callback(['foo'])
+        d2.callback(['bar'])
+
+        # it should be called once more only
+        self.assertEqual(len(called), 2)
+
+    def test_mq_multiple_without_response_having_more_later(self):
+        """Behave correctly on changes burst delayed."""
+        d1 = defer.Deferred()
+        d2 = defer.Deferred()
+        d3 = defer.Deferred()
+        defs = [d1, d2, d3]
+        called = []
+
+        def fake():
+            """Fake."""
+            d = defs[len(called)]
+            called.append(None)
+            return d
+        self.sd.dbus.get_meta_queue = fake
+
+        # call some times, dbus function should be called once
+        self.sd.on_sd_meta_queue_changed()
+        self.sd.on_sd_meta_queue_changed()
+        self.sd.on_sd_meta_queue_changed()
+        self.assertEqual(len(called), 1)
+
+        # first call returns
+        d1.callback(['foo'])
+
+        # while calling second time, generate more requests
+        self.sd.on_sd_meta_queue_changed()
+        self.sd.on_sd_meta_queue_changed()
+        self.sd.on_sd_meta_queue_changed()
+
+        # second call returns
+        d2.callback(['foo'])
+        d3.callback(['foo'])
+
+        # it should be called three times total
+        self.assertEqual(len(called), 3)
+
+    @defer.inlineCallbacks
+    def test_mq_multiple_support_error(self):
+        """Leave the state ok even with an error in the underlying call."""
+        def fake():
+            """Fake."""
+            raise ValueError('foo')
+        self.sd.dbus.get_meta_queue = fake
+
+        # call it, and "absorb" the error we just generated
+        try:
+            yield self.sd.on_sd_meta_queue_changed()
+        except ValueError:
+            pass
+
+        # it should be in IDLE
+        self.assertEqual(self.sd._mq_asking, ASKING_IDLE)
+
 
 class StateTests(unittest.TestCase):
     """Test State class."""
@@ -732,13 +1056,6 @@ class APITests(unittest.TestCase):
                                      False, 'queues', 'connection')
         self.assertTrue(self.called)
 
-    @defer.inlineCallbacks
-    def test_on_initial_data_ready(self):
-        """Called when SD gets all the initial data."""
-        self.flag_called(self.sd, 'on_initial_data_ready_callback')
-        yield self.sd._get_initial_data()
-        self.assertTrue(self.called)
-
 
 class TestLogs(unittest.TestCase):
     """Test logging."""
@@ -746,8 +1063,10 @@ class TestLogs(unittest.TestCase):
     def setUp(self):
         """Set up."""
         self.hdlr = MementoHandler()
-        logging.getLogger('magicicada.syncdaemon').addHandler(self.hdlr)
         self.hdlr.setLevel(logging.DEBUG)
+        logger = logging.getLogger('magicicada.syncdaemon')
+        logger.addHandler(self.hdlr)
+        logger.setLevel(logging.DEBUG)
         self.sd = SyncDaemon(FakeDBusInterface)
 
     def tearDown(self):
@@ -768,8 +1087,12 @@ class TestLogs(unittest.TestCase):
     def test_initial_value(self):
         """Log the initial filling."""
         yield self.sd._get_initial_data()
-        self.assertTrue(self.hdlr.check_info("Getting initial data"))
-        self.assertTrue(self.hdlr.check_info("All initial data is ready"))
+        self.assertTrue(self.hdlr.check_info("Getting offline initial data"))
+        self.assertTrue(self.hdlr.check_info(
+                        "All initial offline data is ready"))
+        self.assertTrue(self.hdlr.check_info("Getting online initial data"))
+        self.assertTrue(self.hdlr.check_info(
+                        "All initial online data is ready"))
 
     def test_start(self):
         """Log the call to start."""
@@ -823,7 +1146,7 @@ class TestLogs(unittest.TestCase):
         self.assertTrue(self.hdlr.check_info(
                         "SD Meta Queue info is new! 1 items"))
 
-    def test_content_queue_changed(self):
+    def test_meta_queue_changed(self):
         """Log that process_cq has new data."""
         self.sd.dbus.get_content_queue = lambda: defer.succeed(['foo'])
         self.sd.on_sd_content_queue_changed()
@@ -857,13 +1180,32 @@ class TestLogs(unittest.TestCase):
         self.sd.on_sd_name_owner_changed(True)
         self.assertTrue(self.hdlr.check_info("SD Name Owner changed: True"))
 
+    def test_on_public_files_list(self):
+        """Log we got a new public files list."""
+        pf1 = PublicFilesData(volume='v', node='n1', path='p', public_url='u')
+        pf2 = PublicFilesData(volume='v', node='n2', path='p', public_url='u')
+        self.sd.on_sd_public_files_list([pf1, pf2])
+        self.assertTrue(self.hdlr.check_info(
+                        "Got new Public Files list (2 items)"))
+
+    def test_on_public_files_changed(self):
+        """Log we got a change in the public files list."""
+        self.sd.public_files = {}
+        pf = PublicFilesData(volume='volume', node='node',
+                             path='path', public_url='url')
+        self.sd.on_sd_public_files_changed(pf, True)
+        self.assertTrue(self.hdlr.check_info(
+                        "Change in Public Files list! is_public=True",
+                        "node=node", "path=u'path'"))
+
 
 class MetadataTests(BaseTest):
     """Get Metadata info."""
 
     def test_get_metadata_no_callback_set(self):
         """It's mandatory to set the callback for this response."""
-        self.assertRaises(ValueError, self.sd.get_metadata, 'path')
+        self.sd.on_metadata_ready_callback()
+        self.assertTrue(self.hdlr.check_warning('on_metadata_ready_callback'))
 
     def test_get_metadata_ok(self):
         """Get the metadata for given path."""
@@ -988,3 +1330,139 @@ class SharesTests(BaseTest):
 
         # test
         self.assertEqual(cal, [2])
+
+
+class PublicFilesTests(BaseTest):
+    """PublicFiles checking."""
+
+    def test_signal_updates_value(self):
+        """Test that when signal arrives it updates the internal attribute."""
+        pf = PublicFilesData(volume='volume', node='node',
+                             path='path', public_url='url')
+        self.sd.on_sd_public_files_list([pf])
+        self.assertEqual(len(self.sd.public_files), 1)
+        pf = self.sd.public_files['node']
+        self.assertEqual(pf.volume, 'volume')
+        self.assertEqual(pf.node, 'node')
+        self.assertEqual(pf.public_url, 'url')
+        self.assertEqual(pf.path, 'path')
+
+    def test_public_files_changed_added_noprevious(self):
+        """Test that it adds the new public file."""
+        # reset and add
+        self.sd.public_files = {}
+        pf = PublicFilesData(volume='volume', node='node',
+                             path='path', public_url='url')
+        self.sd.on_sd_public_files_changed(pf, True)
+
+        # check is there
+        pf = self.sd.public_files['node']
+        self.assertEqual(pf.volume, 'volume')
+        self.assertEqual(pf.node, 'node')
+        self.assertEqual(pf.public_url, 'url')
+        self.assertEqual(pf.path, 'path')
+
+    def test_public_files_changed_added_previous(self):
+        """Test that it updates the public file."""
+        # add one
+        pf = PublicFilesData(volume='volume', node='node',
+                             path='path', public_url='url1')
+        self.sd.on_sd_public_files_changed(pf, True)
+
+        # update it
+        pf = PublicFilesData(volume='volume', node='node',
+                             path='path', public_url='url2')
+        self.sd.on_sd_public_files_changed(pf, True)
+
+        # check is updated
+        pf = self.sd.public_files['node']
+        self.assertEqual(pf.public_url, 'url2')
+
+    def test_public_files_changed_removed_previous(self):
+        """Test that it deletes the public file."""
+        # add one
+        pf = PublicFilesData(volume='volume', node='node',
+                             path='path', public_url='url')
+        self.sd.on_sd_public_files_changed(pf, True)
+
+        # remove it
+        self.sd.on_sd_public_files_changed(pf, False)
+
+        # check is no longer there
+        self.assertFalse('node' in self.sd.public_files)
+
+    def test_public_files_changed_removed_noprevious(self):
+        """Test that support the deletion of anything."""
+        # reset and remove
+        self.sd.public_files = {}
+        pf = PublicFilesData(volume='volume', node='node',
+                             path='path', public_url='url')
+        self.sd.on_sd_public_files_changed(pf, False)
+
+        # check that still is not there
+        self.assertFalse('node' in self.sd.public_files)
+
+
+class HandlingSharesTests(BaseTest):
+    """Handling shares checking."""
+
+    def test_on_share_op_success_callback_set(self):
+        """It's mandatory to set the callback for this response."""
+        self.sd.on_share_op_success_callback()
+        self.assertTrue(self.hdlr.check_warning(
+                                            'on_share_op_success_callback'))
+
+    def test_on_share_op_error_callback_set(self):
+        """It's mandatory to set the callback for this response."""
+        self.sd.on_share_op_error_callback()
+        self.assertTrue(self.hdlr.check_warning('on_share_op_error_callback'))
+
+    def test_accept_share_ok(self):
+        """Accepting a share finishes ok."""
+        # monkeypatch
+        called = []
+        self.sd.dbus.fake_share_response = defer.succeed(None)
+        self.sd.on_share_op_success_callback = lambda sid: called.append(sid)
+        self.sd.on_share_op_error_callback = lambda *a: None
+
+        # execute and test
+        self.sd.accept_share('share_id')
+        self.assertEqual(called, ['share_id'])
+        self.assertTrue(self.hdlr.check_info("Accepting share", "share_id",
+                                             "started"))
+        self.assertTrue(self.hdlr.check_info("Accepting share", "share_id",
+                                             "finished successfully"))
+
+    def test_accept_share_failure(self):
+        """Accepting a share finishes bad."""
+        # monkeypatch
+        called = []
+        e = ShareOperationError(share_id='foo', error='bar')
+        self.sd.dbus.fake_share_response = defer.fail(e)
+        self.sd.on_share_op_error_callback = \
+                                        lambda sid, e: called.append((sid, e))
+        self.sd.on_share_op_success_callback = lambda *a: None
+
+        # execute and test
+        self.sd.accept_share('share_id')
+        self.assertEqual(called, [('share_id', 'bar')])
+        self.assertTrue(self.hdlr.check_info("Accepting share", "share_id",
+                                             "started"))
+        self.assertTrue(self.hdlr.check_info("Accepting share", "share_id",
+                                             "finished with error", "bar"))
+
+    def test_accept_share_error(self):
+        """Really bad error when accepting a share."""
+        # monkeypatch
+        e = ValueError('unexpected failure')
+        self.sd.dbus.fake_share_response = defer.fail(e)
+        self.sd.on_share_op_error_callback = lambda *a: None
+        self.sd.on_share_op_success_callback = lambda *a: None
+
+        # execute and test
+        self.sd.accept_share('share_id')
+        self.assertTrue(self.hdlr.check_info("Accepting share", "share_id",
+                                             "started"))
+        self.assertTrue(self.hdlr.check_error(
+                        "Unexpected error when accepting share", "share_id",
+                        "ValueError", "unexpected failure"))
